@@ -3,60 +3,20 @@ from openbte.heat_source     import HeatSource
 import numpy as np
 from scipy.sparse import linalg 
 import scipy.sparse as sp
-import time
+import openbte.utils as utils
 import matplotlib.pylab as plt
+from cachetools import cached,LRUCache
 from nptyping import NDArray, Shape, Float, Int
 
 
 
-def compute_effective_kappa(T                              ,\
-                            thermal_conductivity           ,\
-                            geometry                       ,\
-                            boundary_conditions            ,\
-                            effective_thermal_conductivity):
-
-
-    if effective_thermal_conductivity.contact == None:
-     return 0
-
-    contact = effective_thermal_conductivity.contact
-
-    bc   = boundary_conditions.get_bc_by_name(contact)
-    if bc == 'Mixed': 
-      sides =   geometry.side_physical_regions[contact]
-    else:  
-      sides =   geometry.periodic_sides[contact]
-
-    v_orth,_ =  geometry.get_decomposed_directions(thermal_conductivity[:geometry.dim,:geometry.dim])
-
-    #Calculate integarted flux
-    P = 0
-    for n,s in enumerate(sides):
-
-        T1 = T[geometry.side_elem_map[s][0]]
-
-        if bc == 'Mixed': 
-            T2   = boundary_conditions.mixed[contact]['value']
-            h    = boundary_conditions.mixed[contact]['boundary_conductance']
-            
-            tmp = v_orth[s] * h/(h + v_orth[s]/geometry.side_areas[s]) 
-
-        if bc == 'Periodic': 
-            T2   = T[geometry.side_elem_map[s][1]] + boundary_conditions.periodic[contact]
-
-            tmp = v_orth[s]
-
-        deltaT = T1 - T2
-
-        P += deltaT*tmp*effective_thermal_conductivity.normalization
-
-    return P
+cache_assembly = LRUCache(maxsize=1e6)
 
 def Fourier(geometry                       : Mesh,\
             thermal_conductivity           : NDArray[Shape["[dim,dim]"], Float],\
             boundary_conditions            : BoundaryConditions,\
             effective_thermal_conductivity : EffectiveThermalConductivity = None,\
-            heat_source                    : HeatSource = None,\
+            heat_source                    : NDArray = [],\
             global_temperature             : NDArray[Shape["[n_volumes]"], Float] = 0,\
             modified_fourier               : bool  = False,\
             fourier_max_iter               : Int   = 40,\
@@ -79,12 +39,16 @@ def Fourier(geometry                       : Mesh,\
   :param verbose: whether to write output on screen
     """
 
-    thermal_conductivity = thermal_conductivity[:geometry.dim,:geometry.dim]
-    v_orth_v,v_nonorth_v =  geometry.get_decomposed_directions(thermal_conductivity)
+    #This specifies the volume to be pinned to zero (for stability)
+    N = 10
 
-    
-    d_vec = [];i_vec = [];j_vec = []
-    for s in geometry.internal:
+    #@cached(cache=cache_assembly)
+    def assembly(kappa,modified_fourier):
+
+     v_orth_v,v_nonorth_v =  geometry.get_decomposed_directions(kappa)
+
+     d_vec = [];i_vec = [];j_vec = []
+     for s in geometry.internal:
        c = geometry.side_centroids[s]
        #---------------------------- 
        i,j  = geometry.side_elem_map[s] 
@@ -98,16 +62,17 @@ def Fourier(geometry                       : Mesh,\
        i_vec.append(j); j_vec.append(j); d_vec.append( v_orth/vj)
        i_vec.append(j); j_vec.append(i); d_vec.append(-v_orth/vj)
 
-    #Boundary condition
-    B = np.zeros(geometry.n_elems)
 
-    #Dirichlet
-    for key,contact in boundary_conditions.mixed.items():
+     #Boundary condition
+     B = np.zeros(geometry.n_elems)
+
+     #Dirichlet
+     for key,contact in boundary_conditions.mixed.items():
+
         #Get all the sides given a key
         sides = geometry.side_physical_regions[key]
 
         for s in sides: 
-
          i          = geometry.side_elem_map[s][0]   
          vi         = geometry.elem_volumes[i]
 
@@ -120,8 +85,8 @@ def Fourier(geometry                       : Mesh,\
          i_vec.append(i); j_vec.append(i); d_vec.append( tmp/vi)
          B[i] = contact['value'] * tmp/vi
 
-    #Periodic
-    for region,jump in boundary_conditions.periodic.items():
+     #Periodic
+     for region,jump in boundary_conditions.periodic.items():
 
         area = 0 
         for s in geometry.periodic_sides[region]:
@@ -132,46 +97,62 @@ def Fourier(geometry                       : Mesh,\
          B[i]      += jump * v_orth_v[s]/vi
          B[j]      -= jump * v_orth_v[s]/vj
 
-    #Add heat source
-    if not heat_source == None:
-       B += heat_source.variables['Heat_Source']['data']
+     #Add heat source
+     if not len(heat_source) == 0 :
+      if not modified_fourier:
+       #B += heat_source.variables['Heat_Source']['data']*heat_source.aux['heat_source_coeff'] #Heat source is in K (like as in the BTE)
+       #else:
+       B += heat_source*1e-18 #from W/m^3 tp W/nm^2/m [kappa is still in W/m/K]
+      else: 
+       B += heat_source #K
 
-    F   = sp.csc_matrix((d_vec,(i_vec,j_vec)),shape = (geometry.n_elems,geometry.n_elems))
+     F   = sp.csc_matrix((d_vec,(i_vec,j_vec)),shape = (geometry.n_elems,geometry.n_elems))
 
+     #modified Fourier
+     if modified_fourier:
+      F += sp.eye(geometry.n_elems,format='csc')
+     else:   
+      if not effective_thermal_conductivity == None:
+       #Fix a point for stability
+       scale = np.ones(geometry.n_elems); scale[N] = 0
+       F.data = F.data * scale[F.indices]
+       F[N,N] = 1
+       #-----------------------------
 
-    #modified Fourier
+     lu = linalg.splu(F)
+
+     #Prepare data for nonorthogonal contribution
+     inds  = geometry.side_elem_map[geometry.internal]
+     w     = geometry.interp_weights[geometry.internal]
+     vi    = geometry.elem_volumes[inds[:,0]]
+     vj    = geometry.elem_volumes[inds[:,1]]
+     v_nonorth = v_nonorth_v[geometry.internal]
+     #------------------------------------
+
+     return lu,B,w,inds,v_nonorth,vi,vj
+
+    thermal_conductivity = thermal_conductivity[:geometry.dim,:geometry.dim]
+
+    lu,B,w,inds,v_nonorth,vi,vj = assembly(thermal_conductivity,modified_fourier)
+
+   
     if modified_fourier:
-       F += sp.eye(geometry.n_elems,format='csc')
        B += global_temperature
-
-    #Fix a point for stability
-    N = 10
-    F[:,N] = 0
-    F[N,N] = 1
-    B[N] = 0
-    #-----
-
-    #Create super lu -------
-    lu  = linalg.splu(F)
+    else:   
+       B[N] = 0
     #-----------------------
 
-    #Prepare data for nonorthogonal contribution
-    inds  = geometry.side_elem_map[geometry.internal]
-    w     = geometry.interp_weights[geometry.internal]
-    vi    = geometry.elem_volumes[inds[:,0]]
-    vj    = geometry.elem_volumes[inds[:,1]]
-    v_nonorth = v_nonorth_v[geometry.internal]
-    #------------------------------------
-
-
     C = np.zeros_like(B); norm_C_old = 0
     error = 1
     n_iter = 0
-    
+    kappa_eff = 0
     while error > fourier_rtol and n_iter < fourier_max_iter:
 
         n_iter +=1
         T        = lu.solve(B+C)
+        #Normalized T 
+        if (not effective_thermal_conductivity == None) and (not modified_fourier == True):
+         T -= (max(T) + min(T))*0.5
         
         gradient = geometry.gradient(boundary_conditions,T)
        
@@ -188,12 +169,9 @@ def Fourier(geometry                       : Mesh,\
         error  = 0 if norm_C == 0 else abs(norm_C-norm_C_old)/norm_C
         norm_C_old = norm_C
 
-        kappa_eff = compute_effective_kappa(T,\
-                                              thermal_conductivity,\
-                                              geometry,\
-                                              boundary_conditions,\
-                                              effective_thermal_conductivity)
-
+        if not effective_thermal_conductivity == None:
+            kappa_eff = effective_thermal_conductivity.get_diffusive(T,geometry,boundary_conditions,thermal_conductivity)
+     
         if verbose:
          strc = "Kappa (Fourier): {:E}, Iterations {:n}, Error {:E}".format(kappa_eff,n_iter,error) 
          print(strc)
@@ -202,10 +180,6 @@ def Fourier(geometry                       : Mesh,\
 
     J = -np.einsum('ij,cj->ci',thermal_conductivity,gradient)
 
-    #Normalized T 
-    if not effective_thermal_conductivity.contact == None:
-      T -= (max(T) + min(T))*0.5
-
     variables = {'Temperature_Fourier':{'data':T,'units':'K'},'Flux_Fourier':{'data':J,'units':'W/m/m'}}
     return SolverResults(kappa_eff = kappa_eff,variables=variables)
 
